//-----------------------------------------------------------------------------
// Title      : bsp_ub024_mk3_ext_pca9555
//-----------------------------------------------------------------------------
// Funktion   : Externes TWI-Modul : 16bit IO/PortExpander
// Schaltung  : myAVR MK3 + myTWI-AddOn-PortExpander
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 20.11.2010
// Version    : 1.1
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <ub_ext_module/EXT_TWI_PCA9555.h>
#include <ub_mk3_2560/MK3_2560_LED.h>

//-----------------------------------------------------------------------------
// Hinweis :
// > Quick-Jumper Port-L muss gesetzt sein
// > externes TWI-Modul muss am Stecker rechts unten angeschlossen sein
// > TWI-Port-Pins (D0+D1) werden benoetigt
// > Das PCA9555 muss auf ADR 40h eingestellt sein			
// - das Programm liest zyklisch den externen Port-A ein
//   und gibt das Ergebnis an den LEDs wieder aus
// - bei einem Fehler wird 0x55 angezeigt
//-----------------------------------------------------------------------------
int main (void)
{
	char wert=0;
	unsigned int p=0;

	MK3_INIT_LED(); // init der LEDs
	EXT_INIT_PCA9555(); // init vom externen PCA9555
	
	// externes Port-A als 8bit Eingang definieren
	EXT_PCA9555_CONF_PORTA(ADR0_PCA9555,PORT_IN);

	while (true)
	{
		// einlesen vom externen Port-A
		wert=EXT_PCA9555_READ_PORTA(ADR0_PCA9555);
		
		if(EXT_PCA9555_OK==true) {
		  // wenn alles ok
		  
		  // anzeigen des wertes
		  MK3_LED_ZAHL(wert);
		}
		else {
	      // 0x55 wird als FEHLER angezeigt
	      MK3_LED_ZAHL(0x55);		
		}
		
	    for(p=0;p<60000;p++) {
		// kleine Pause
	    }

	}
	return 0;
}
//-----------------------------------------------------------------------------
