//-----------------------------------------------------------------------------
// EXT_TWI_PCA9555
// Version 1.2
// 10.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer ein externes Modul
// mit dem TWI-IC : PCA9555
// 16Bit IO-Port
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/"
// liegen und mit include <UB_EXT_MODULE/EXT_TWI_PCA9555.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : TWI-BUS
// Benutzt : MK3_2560_TWI.h = Port-D (Bit0 bis Bit1)
//-----------------------------------------------------------------------------
//
// Info zum PCA9555
// Grundadresse = 0x40 (wenn keine Pull-UPs aktiv sind)
// 3 Adr-Pins   = Bit1 + Bit2 + Bi3
// Dadurch 8 moegliche Konfigurationen : 40, 42, 44, 46, 48, 4A, 4C, 4E
//
// Die I/O-Konfiguration kann bei allen 16 Port-Pins frei belegt werden
// Konfiguration nach Reset : alle 16 Pins = Eingang
//-----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include "ub_mk3_2560/MK3_2560_TWI.h" //  bindet die TWI-Funktionen ein


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "EXT_INIT_PCA9555" starten
// 02. Funktion "EXT_PCA9555_CONF_PORTA/B" zum einstellen der Pin-Konfiguration
// 02A. Die Variable : EXT_PCA9555_OK zeigt an ob OK oder FEHLER
// 03. Funktion "EXT_PCA9555_READ_PORTA/B" zum einlesen eines Ports
// 03A. Die Variable : EXT_PCA9555_OK zeigt an ob OK oder FEHLER
// 04. Funktion "EXT_PCA9555_WRITE_PORTA/B" zum schreiben eines Ports
// 04A. Die Variable : EXT_PCA9555_OK zeigt an ob OK oder FEHLER
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------

typedef enum {
  ADR0_PCA9555 = 0x40,
  ADR1_PCA9555 = 0x42,
  ADR2_PCA9555 = 0x44,
  ADR3_PCA9555 = 0x46,
  ADR4_PCA9555 = 0x48,
  ADR5_PCA9555 = 0x4A,
  ADR6_PCA9555 = 0x4C,
  ADR7_PCA9555 = 0x4E
}ADR_PCA9555;

typedef enum {
  PORT_OUT=0x00, // alle 8bit als Ausgang
  PORT_IN=0xFF,  // alle 8bit als Eingang
  OUT_IN=0x0F,   // 4 Ausgaenge + 4 Eingaenge -> Hi-Nibble=OUT, Lo-Nibble=IN
  IN_OUT=0xF0    // 4 Ausgaenge + 4 Eingaenge -> Hi-Nibble=IN , Lo-Nibble=OUT
}IO_PCA9555;


//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void EXT_INIT_PCA9555(void);
void EXT_PCA9555_CONF_PORTA(ADR_PCA9555 twi_adr,char io_conf);
void EXT_PCA9555_CONF_PORTB(ADR_PCA9555 twi_adr,char io_conf);
char EXT_PCA9555_READ_PORTA(ADR_PCA9555 twi_adr);
char EXT_PCA9555_READ_PORTB(ADR_PCA9555 twi_adr);
void EXT_PCA9555_WRITE_PORTA(ADR_PCA9555 twi_adr, char wert);
void EXT_PCA9555_WRITE_PORTB(ADR_PCA9555 twi_adr, char wert);


//-----------------------------------------------------------------------------
// Globale Variabeln
//-----------------------------------------------------------------------------
bool EXT_PCA9555_OK=false;        // zum ueberpruefen der fehlerfreien Funktion

//-----------------------------------------------------------------------------
// Register-Definitionen
//-----------------------------------------------------------------------------
#define PCA9555_REG_IN_A   0x00	// Register-Adresse von Port-A (IN)
#define PCA9555_REG_IN_B   0x01	// Register-Adresse von Port-B (IN)
#define PCA9555_REG_OUT_A  0x02	// Register-Adresse von Port-A (OUT)
#define PCA9555_REG_OUT_B  0x03	// Register-Adresse von Port-B (OUT)
#define PCA9555_REG_DDR_A  0x06	// Register-Adresse von Port-A (DataDirection)
#define PCA9555_REG_DDR_B  0x07	// Register-Adresse von Port-B (DataDirection)


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
void EXT_PCA9555_WRITE_REG(ADR_PCA9555 twi_adr, char adr, char wert);
char EXT_PCA9555_READ_REG(ADR_PCA9555 twi_adr, char adr);


//-----------------------------------------------------------------------------
void EXT_INIT_PCA9555(void) {	// zum initialisieren des externen PCA9555

  // init vom TWI-BUS
  MK3_INIT_TWI();
}


//-----------------------------------------------------------------------------
void EXT_PCA9555_CONF_PORTA(ADR_PCA9555 twi_adr,char io_conf) {
  // setzt Konfiguration von Port-A
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt sonst auf TRUE  
  
  EXT_PCA9555_OK=true; // zuerst auf OK (Fehler wird vom UP gesetzt)
  
  // schreiben des Konfig-Wertes
  EXT_PCA9555_WRITE_REG(twi_adr,PCA9555_REG_DDR_A,io_conf);   
 
}

//-----------------------------------------------------------------------------
void EXT_PCA9555_CONF_PORTB(ADR_PCA9555 twi_adr,char io_conf) {
  // setzt Konfiguration von Port-B
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt sonst auf TRUE
  
  EXT_PCA9555_OK=true; // zuerst auf OK (Fehler wird vom UP gesetzt)
  
  // schreiben des Konfig-Wertes
  EXT_PCA9555_WRITE_REG(twi_adr,PCA9555_REG_DDR_B,io_conf); 
}

//-----------------------------------------------------------------------------
char EXT_PCA9555_READ_PORTA(ADR_PCA9555 twi_adr) {
  // auslesen von Port-A
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt sonst auf TRUE

  char ret_wert=0;
  
  EXT_PCA9555_OK=true; // zuerst auf OK (Fehler wird vom UP gesetzt)
  
  // auslesen des Port-Wertes
  ret_wert=EXT_PCA9555_READ_REG(twi_adr,PCA9555_REG_IN_A);  
  
  return(ret_wert);  
}

//-----------------------------------------------------------------------------
char EXT_PCA9555_READ_PORTB(ADR_PCA9555 twi_adr) {
  // auslesen von Port-B
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt sonst auf TRUE

  char ret_wert=0;
  
  EXT_PCA9555_OK=true; // zuerst auf OK (Fehler wird vom UP gesetzt)
  
  // auslesen des Port-Wertes
  ret_wert=EXT_PCA9555_READ_REG(twi_adr,PCA9555_REG_IN_B);  
  
  return(ret_wert);  
}

//-----------------------------------------------------------------------------
void EXT_PCA9555_WRITE_PORTA(ADR_PCA9555 twi_adr, char wert) {
  // beschreiben von Port-A
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt sonst auf TRUE 
  
  EXT_PCA9555_OK=true; // zuerst auf OK (Fehler wird vom UP gesetzt)
  
  // schreiben des Port-Wertes
  EXT_PCA9555_WRITE_REG(twi_adr,PCA9555_REG_OUT_A,wert);
}

//-----------------------------------------------------------------------------
void EXT_PCA9555_WRITE_PORTB(ADR_PCA9555 twi_adr, char wert) {
  // beschreiben von Port-B
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt sonst auf TRUE 
  
  EXT_PCA9555_OK=true; // zuerst auf OK (Fehler wird vom UP gesetzt)
  
  // schreiben des Port-Wertes
  EXT_PCA9555_WRITE_REG(twi_adr,PCA9555_REG_OUT_B,wert);
}

//-----------------------------------------------------------------------------
void EXT_PCA9555_WRITE_REG(ADR_PCA9555 twi_adr, char adr, char wert) { // INTERNE-FUNKTION
  // beschreibt ein Register vom PCA9555
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt 
    
  bool ok=false;
  char antwort=0;
  
  ok=MK3_TWI_START();	        // sendet Start-Kennung
  if(ok==true) {
    // nur wenn Startkennung erfolgreich
    antwort=MK3_TWI_SEND(twi_adr,false);        // 1.Byte = Slave-Adresse (ohne ACK)
    if(antwort==TWSR_SLAVE_OK) {
      // Slave hat mit ACK geantwortet 
      MK3_TWI_SEND(adr,false);                  // 2.Byte = Register (ohne ACK)
      MK3_TWI_SEND(wert,false);                 // 3.Byte = Wert (ohne ACK)
      ok=true;
    }
    else {
      // Slave hat nicht geantwortet 
      ok=false;      
    }
    MK3_TWI_STOP();		         // sendet Stop-Kennung 
    MK3_TWI_WAIT(10);                    // kurze Pause (ca. 5ms) WICHTIG !!
  }      
  else {
    // Startkennung falsch
    ok=false;    
  }  

  // einen Fehler speichern
  if(ok==false) EXT_PCA9555_OK=false;
}


//-----------------------------------------------------------------------------
char EXT_PCA9555_READ_REG(ADR_PCA9555 twi_adr, char adr) { // INTERNE-FUNKTION
  // liest ein Register vom PCA9555
  // 
  // bei einem Fehler wird EXT_PCA9555_OK auf FALSE gesetzt
    
  char ret_wert=0,antwort=0;
  bool ok=false;
  
  ok=MK3_TWI_START();		           // sendet Start-Kennung
  if(ok==true) {
    // nur wenn Startkennung erfolgreich    
    antwort=MK3_TWI_SEND(twi_adr,false);   // 1.Byte = Slave-Adresse (ohne ACK)
    if(antwort==TWSR_SLAVE_OK) {
      // Slave hat mit ACK geantwortet
      MK3_TWI_SEND(adr,false);	           // 2.Byte = Adresse (ohne ACK)
      MK3_TWI_START();		           // sendet Start-Kennung
      MK3_TWI_SEND(twi_adr+1,false);	   // 1.Byte = Slave-Adresse+1 (ohne ACK)
      ret_wert=MK3_TWI_READ(false);        // 2.Byte wird vom Slave gesendet (ohne ACK)
      ok=true;
    }
    else {
      // Slave hat nicht geantwortet 
      ok=false;
    }
    MK3_TWI_STOP();		           // sendet Stop-Kennung 
    MK3_TWI_WAIT(10);                      // kurze Pause (ca. 5ms) WICHTIG !!
  }
  else {
    // Startkennung falsch
    ok=false;
  }  
  
  // einen Fehler speichern und 0 zurueckgeben
  if(ok==false) {
    EXT_PCA9555_OK=false;  
    ret_wert=0;
  }
  
  return(ret_wert);
}
